@@ -1,6 +1,6 @@
 -- |
 -- Module      :  Text.Megaparsec.Char.Lexer
--- Copyright   :  © 2015–2018 Megaparsec contributors
+-- Copyright   :  © 2015–2019 Megaparsec contributors
 --                © 2007 Paolo Martini
 --                © 1999–2001 Daan Leijen
 -- License     :  FreeBSD
@@ -11,8 +11,7 @@
 --
 -- High-level parsers to help you write your lexer. The module doesn't
 -- impose how you should write your parser, but certain approaches may be
--- more elegant than others. Especially important theme is parsing of white
--- space, comments, and indentation.
+-- more elegant than others.
 --
 -- Parsing of white space is an important part of any parser. We propose a
 -- convention where __every lexeme parser assumes no spaces before the__
@@ -28,7 +27,6 @@
 --
 -- To do lexing of byte streams, see "Text.Megaparsec.Byte.Lexer".
 
-{-# LANGUAGE CPP                 #-}
 {-# LANGUAGE FlexibleContexts    #-}
 {-# LANGUAGE MultiWayIf          #-}
 {-# LANGUAGE ScopedTypeVariables #-}
@@ -55,6 +53,7 @@ module Text.Megaparsec.Char.Lexer
   , charLiteral
     -- * Numbers
   , decimal
+  , binary
   , octal
   , hexadecimal
   , scientific
@@ -70,7 +69,7 @@ import Data.Maybe (listToMaybe, fromMaybe, isJust)
 import Data.Proxy
 import Data.Scientific (Scientific)
 import Text.Megaparsec
-import qualified Data.CaseInsensitive as CI
+import Text.Megaparsec.Lexer
 import qualified Data.Char            as Char
 import qualified Data.Scientific      as Sci
 import qualified Data.Set             as E
@@ -79,87 +78,6 @@ import qualified Text.Megaparsec.Char as C
 ----------------------------------------------------------------------------
 -- White space
 
--- | @'space' sc lineComment blockComment@ produces parser that can parse
--- white space in general. It's expected that you create such a parser once
--- and pass it to other functions in this module as needed (when you see
--- @spaceConsumer@ in documentation, usually it means that something like
--- 'space' is expected there).
---
--- @sc@ is used to parse blocks of space characters. You can use 'C.space1'
--- from "Text.Megaparsec.Char" for this purpose as well as your own parser
--- (if you don't want to automatically consume newlines, for example). Make
--- sure the parser does not succeed on empty input though. In earlier
--- version 'C.spaceChar' was recommended, but now parsers based on
--- 'takeWhile1P' are preferred because of their speed.
---
--- @lineComment@ is used to parse line comments. You can use
--- 'skipLineComment' if you don't need anything special.
---
--- @blockComment@ is used to parse block (multi-line) comments. You can use
--- 'skipBlockComment' or 'skipBlockCommentNested' if you don't need anything
--- special.
---
--- If you don't want to allow a kind of comment, simply pass 'empty' which
--- will fail instantly when parsing of that sort of comment is attempted and
--- 'space' will just move on or finish depending on whether there is more
--- white space for it to consume.
-
-space :: MonadParsec e s m
-  => m () -- ^ A parser for space characters which does not accept empty
-          -- input (e.g. 'C.space1')
-  -> m () -- ^ A parser for a line comment (e.g. 'skipLineComment')
-  -> m () -- ^ A parser for a block comment (e.g. 'skipBlockComment')
-  -> m ()
-space sp line block = skipMany $ choice
-  [hidden sp, hidden line, hidden block]
-{-# INLINEABLE space #-}
-
--- | This is a wrapper for lexemes. Typical usage is to supply the first
--- argument (parser that consumes white space, probably defined via 'space')
--- and use the resulting function to wrap parsers for every lexeme.
---
--- > lexeme  = L.lexeme spaceConsumer
--- > integer = lexeme L.decimal
-
-lexeme :: MonadParsec e s m
-  => m ()              -- ^ How to consume white space after lexeme
-  -> m a               -- ^ How to parse actual lexeme
-  -> m a
-lexeme spc p = p <* spc
-{-# INLINEABLE lexeme #-}
-
--- | This is a helper to parse symbols, i.e. verbatim strings. You pass the
--- first argument (parser that consumes white space, probably defined via
--- 'space') and then you can use the resulting function to parse strings:
---
--- > symbol    = L.symbol spaceConsumer
--- >
--- > parens    = between (symbol "(") (symbol ")")
--- > braces    = between (symbol "{") (symbol "}")
--- > angles    = between (symbol "<") (symbol ">")
--- > brackets  = between (symbol "[") (symbol "]")
--- > semicolon = symbol ";"
--- > comma     = symbol ","
--- > colon     = symbol ":"
--- > dot       = symbol "."
-
-symbol :: MonadParsec e s m
-  => m ()              -- ^ How to consume white space after lexeme
-  -> Tokens s          -- ^ Symbol to parse
-  -> m (Tokens s)
-symbol spc = lexeme spc . C.string
-{-# INLINEABLE symbol #-}
-
--- | Case-insensitive version of 'symbol'. This may be helpful if you're
--- working with case-insensitive languages.
-
-symbol' :: (MonadParsec e s m, CI.FoldCase (Tokens s))
-  => m ()              -- ^ How to consume white space after lexeme
-  -> Tokens s          -- ^ Symbol to parse (case-insensitive)
-  -> m (Tokens s)
-symbol' spc = lexeme spc . C.string'
-{-# INLINEABLE symbol' #-}
-
 -- | Given comment prefix this function returns a parser that skips line
 -- comments. Note that it stops just before the newline character but
 -- doesn't consume the newline. Newline is either supposed to be consumed by
@@ -179,7 +97,7 @@ skipBlockComment :: (MonadParsec e s m, Token s ~ Char)
   => Tokens s          -- ^ Start of block comment
   -> Tokens s          -- ^ End of block comment
   -> m ()
-skipBlockComment start end = p >> void (manyTill C.anyChar n)
+skipBlockComment start end = p >> void (manyTill anySingle n)
   where
     p = C.string start
     n = C.string end
@@ -196,7 +114,7 @@ skipBlockCommentNested :: (MonadParsec e s m, Token s ~ Char)
   -> m ()
 skipBlockCommentNested start end = p >> void (manyTill e n)
   where
-    e = skipBlockCommentNested start end <|> void C.anyChar
+    e = skipBlockCommentNested start end <|> void anySingle
     p = C.string start
     n = C.string end
 {-# INLINEABLE skipBlockCommentNested #-}
@@ -213,8 +131,8 @@ skipBlockCommentNested start end = p >> void (manyTill e n)
 -- @since 4.3.0
 
 indentLevel :: MonadParsec e s m => m Pos
-indentLevel = sourceColumn <$> getPosition
-{-# INLINEABLE indentLevel #-}
+indentLevel = sourceColumn <$> getSourcePos
+{-# INLINE indentLevel #-}
 
 -- | Fail reporting incorrect indentation error. The error has attached
 -- information:
@@ -271,8 +189,8 @@ nonIndented :: MonadParsec e s m
 nonIndented sc p = indentGuard sc EQ pos1 *> p
 {-# INLINEABLE nonIndented #-}
 
--- | The data type represents available behaviors for parsing of indented
--- tokens. This is used in 'indentBlock', which see.
+-- | Behaviors for parsing of indented tokens. This is used in
+-- 'indentBlock', which see.
 --
 -- @since 4.3.0
 
@@ -308,7 +226,7 @@ indentBlock sc r = do
   ref <- indentLevel
   a   <- r
   case a of
-    IndentNone x -> sc *> return x
+    IndentNone x -> x <$ sc
     IndentMany indent f p -> do
       mlvl <- (optional . try) (C.eol *> indentGuard sc GT ref)
       done <- isJust <$> optional eof
@@ -350,9 +268,9 @@ indentedItems ref lvl sc p = go
 -- | Create a parser that supports line-folding. The first argument is used
 -- to consume white space between components of line fold, thus it /must/
 -- consume newlines in order to work properly. The second argument is a
--- callback that receives a custom space-consuming parser as argument. This
--- parser should be used after separate components of line fold that can be
--- put on different lines.
+-- callback that receives a custom space-consuming parser as an argument.
+-- This parser should be used after separate components of line fold that
+-- can be put on different lines.
 --
 -- An example should clarify the usage pattern:
 --
@@ -397,9 +315,9 @@ charLiteral :: (MonadParsec e s m, Token s ~ Char) => m Char
 charLiteral = label "literal character" $ do
   -- The @~@ is needed to avoid requiring a MonadFail constraint,
   -- and we do know that r will be non-empty if count' succeeds.
-  r <- lookAhead (count' 1 8 C.anyChar)
+  r <- lookAhead (count' 1 10 anySingle)
   case listToMaybe (Char.readLitChar r) of
-    Just (c, r') -> c <$ skipCount (length r - length r') C.anyChar
+    Just (c, r') -> c <$ skipCount (length r - length r') anySingle
     Nothing      -> unexpected (Tokens (head r:|[]))
 {-# INLINEABLE charLiteral #-}
 
@@ -409,14 +327,12 @@ charLiteral = label "literal character" $ do
 -- | Parse an integer in decimal representation according to the format of
 -- integer literals described in the Haskell report.
 --
--- If you need to parse signed integers, see 'signed' combinator.
+-- If you need to parse signed integers, see the 'signed' combinator.
 --
 -- __Note__: before version 6.0.0 the function returned 'Integer', i.e. it
 -- wasn't polymorphic in its return type.
 
-decimal
-  :: forall e s m a. (MonadParsec e s m, Token s ~ Char, Integral a)
-  => m a
+decimal :: (MonadParsec e s m, Token s ~ Char, Integral a) => m a
 decimal = decimal_ <?> "integer"
 {-# INLINEABLE decimal #-}
 
@@ -429,6 +345,28 @@ decimal_ = mkNum <$> takeWhile1P (Just "digit") Char.isDigit
   where
     mkNum    = foldl' step 0 . chunkToTokens (Proxy :: Proxy s)
     step a c = a * 10 + fromIntegral (Char.digitToInt c)
+{-# INLINE decimal_ #-}
+
+-- | Parse an integer in binary representation. Binary number is expected to
+-- be a non-empty sequence of zeroes “0” and ones “1”.
+--
+-- You could of course parse some prefix before the actual number:
+--
+-- > binary = char '0' >> char' 'b' >> L.binary
+--
+-- @since 7.0.0
+
+binary
+  :: forall e s m a. (MonadParsec e s m, Token s ~ Char, Integral a)
+  => m a
+binary = mkNum
+  <$> takeWhile1P Nothing isBinDigit
+  <?> "binary integer"
+  where
+    mkNum        = foldl' step 0 . chunkToTokens (Proxy :: Proxy s)
+    step a c     = a * 2 + fromIntegral (Char.digitToInt c)
+    isBinDigit x = x == '0' || x == '1'
+{-# INLINEABLE binary #-}
 
 -- | Parse an integer in octal representation. Representation of octal
 -- number is expected to be according to the Haskell report except for the
