@@ -1,6 +1,6 @@
 -- |
 -- Module      :  Text.Megaparsec.Class
--- Copyright   :  © 2015–2018 Megaparsec contributors
+-- Copyright   :  © 2015–2019 Megaparsec contributors
 --                © 2007 Paolo Martini
 --                © 1999–2001 Daan Leijen
 -- License     :  FreeBSD
@@ -25,7 +25,6 @@ module Text.Megaparsec.Class
   ( MonadParsec (..) )
 where
 
-import Control.Applicative
 import Control.Monad
 import Control.Monad.Identity
 import Control.Monad.Trans
@@ -45,10 +44,6 @@ import qualified Control.Monad.Trans.Writer.Strict as S
 import Control.Monad.Trans.Identity
 #endif
 
-#if !MIN_VERSION_base(4,8,0)
-import Data.Monoid
-#endif
-
 -- | Type class describing monads that implement the full set of primitive
 -- parsers.
 --
@@ -56,8 +51,7 @@ import Data.Monoid
 -- taken advantage of as much as possible if your aim is a fast parser:
 -- 'tokens', 'takeWhileP', 'takeWhile1P', and 'takeP'.
 
-class (Stream s, Alternative m, MonadPlus m)
-    => MonadParsec e s m | m -> e s where
+class (Stream s, MonadPlus m) => MonadParsec e s m | m -> e s where
 
   -- | The most general way to stop parsing and report a trivial
   -- 'ParseError'.
@@ -70,7 +64,8 @@ class (Stream s, Alternative m, MonadPlus m)
     -> m a
 
   -- | The most general way to stop parsing and report a fancy 'ParseError'.
-  -- To report a single custom parse error, see 'customFailure'.
+  -- To report a single custom parse error, see
+  -- 'Text.Megaparsec.customFailure'.
   --
   -- @since 6.0.0
 
@@ -125,11 +120,12 @@ class (Stream s, Alternative m, MonadPlus m)
   -- unexpected "le"
   -- expecting "let" or "lexical"
   --
-  -- __Please note__ that as of Megaparsec 4.4.0, 'string' backtracks
-  -- automatically (see 'tokens'), so it does not need 'try'. However, the
-  -- examples above demonstrate the idea behind 'try' so well that it was
-  -- decided to keep them. You still need to use 'try' when your
-  -- alternatives are complex, composite parsers.
+  -- __Please note__ that as of Megaparsec 4.4.0,
+  -- 'Text.Megaparsec.Char.string' backtracks automatically (see 'tokens'),
+  -- so it does not need 'try'. However, the examples above demonstrate the
+  -- idea behind 'try' so well that it was decided to keep them. You still
+  -- need to use 'try' when your alternatives are complex, composite
+  -- parsers.
 
   try :: m a -> m a
 
@@ -160,12 +156,12 @@ class (Stream s, Alternative m, MonadPlus m)
   -- @since 4.4.0
 
   withRecovery
-    :: (ParseError (Token s) e -> m a) -- ^ How to recover from failure
+    :: (ParseError s e -> m a) -- ^ How to recover from failure
     -> m a             -- ^ Original parser
     -> m a             -- ^ Parser that can recover from failures
 
   -- | @'observing' p@ allows to “observe” failure of the @p@ parser, should
-  -- it happen, without actually ending parsing, but instead getting the
+  -- it happen, without actually ending parsing but instead getting the
   -- 'ParseError' in 'Left'. On success parsed value is returned in 'Right'
   -- as usual. Note that this primitive just allows you to observe parse
   -- errors as they happen, it does not backtrack or change how the @p@
@@ -175,44 +171,41 @@ class (Stream s, Alternative m, MonadPlus m)
 
   observing
     :: m a             -- ^ The parser to run
-    -> m (Either (ParseError (Token s) e) a)
+    -> m (Either (ParseError s e) a)
 
-  -- | This parser only succeeds at the end of the input.
+  -- | This parser only succeeds at the end of input.
 
   eof :: m ()
 
-  -- | The parser @'token' test mrep@ accepts a token @t@ with result @x@
-  -- when the function @test t@ returns @'Right' x@. @mrep@ may provide
-  -- representation of the token to report in error messages when input
-  -- stream in empty.
+  -- | The parser @'token' test expected@ accepts a token @t@ with result
+  -- @x@ when the function @test t@ returns @'Just' x@. @expected@ specifies
+  -- the collection of expected items to report in error messages.
   --
   -- This is the most primitive combinator for accepting tokens. For
-  -- example, the 'Text.Megaparsec.Char.satisfy' parser is implemented as:
+  -- example, the 'Text.Megaparsec.satisfy' parser is implemented as:
   --
-  -- > satisfy f = token testChar Nothing
+  -- > satisfy f = token testToken E.empty
   -- >   where
-  -- >     testChar x =
-  -- >       if f x
-  -- >         then Right x
-  -- >         else Left (pure (Tokens (x:|[])), Set.empty)
+  -- >     testToken x = if f x then Just x else Nothing
+  --
+  -- __Note__: type signature of this primitive was changed in the version
+  -- /7.0.0/.
 
   token
-    :: (Token s -> Either ( Maybe (ErrorItem (Token s))
-                          , Set (ErrorItem (Token s)) ) a)
-       -- ^ Matching function for the token to parse, it allows to construct
-       -- arbitrary error message on failure as well; things in the tuple
-       -- are: unexpected item (if any) and expected items
-    -> Maybe (Token s) -- ^ Token to report when input stream is empty
+    :: (Token s -> Maybe a)
+       -- ^ Matching function for the token to parse
+    -> Set (ErrorItem (Token s))
+       -- ^ Expected items (in case of an error)
     -> m a
 
-  -- | The parser @'tokens' test@ parses a chunk of input and returns it.
-  -- Supplied predicate @test@ is used to check equality of given and parsed
-  -- chunks after a candidate chunk of correct length is fetched from the
-  -- stream.
+  -- | The parser @'tokens' test chk@ parses a chunk of input @chk@ and
+  -- returns it. The supplied predicate @test@ is used to check equality of
+  -- given and parsed chunks after a candidate chunk of correct length is
+  -- fetched from the stream.
   --
-  -- This can be used for example to write 'Text.Megaparsec.Char.string':
+  -- This can be used for example to write 'Text.Megaparsec.chunk':
   --
-  -- > string = tokens (==)
+  -- > chunk = tokens (==)
   --
   -- Note that beginning from Megaparsec 4.4.0, this is an auto-backtracking
   -- primitive, which means that if it fails, it never consumes any input.
@@ -239,15 +232,15 @@ class (Stream s, Alternative m, MonadPlus m)
 
   -- | Parse /zero/ or more tokens for which the supplied predicate holds.
   -- Try to use this as much as possible because for many streams the
-  -- combinator is much faster than parsers built with 'many' and
-  -- 'Text.Megaparsec.Char.satisfy'.
+  -- combinator is much faster than parsers built with
+  -- 'Control.Monad.Combinators.many' and 'Text.Megaparsec.satisfy'.
   --
   -- The following equations should clarify the behavior:
   --
   -- > takeWhileP (Just "foo") f = many (satisfy f <?> "foo")
   -- > takeWhileP Nothing      f = many (satisfy f)
   --
-  -- The combinator never fails, although it may parse an empty chunk.
+  -- The combinator never fails, although it may parse the empty chunk.
   --
   -- @since 6.0.0
 
