@@ -1,6 +1,6 @@
 -- |
 -- Module      :  Text.Megaparsec.Error.Builder
--- Copyright   :  © 2015–2018 Megaparsec contributors
+-- Copyright   :  © 2015–2019 Megaparsec contributors
 -- License     :  FreeBSD
 --
 -- Maintainer  :  Mark Karpov <markkarpov92@gmail.com>
@@ -8,23 +8,22 @@
 -- Portability :  portable
 --
 -- A set of helpers that should make construction of 'ParseError's more
--- concise. This is primarily useful in test suites and for debugging, you
--- most certainly don't need it for normal usage.
+-- concise. This is primarily useful in test suites and for debugging.
 --
 -- @since 6.0.0
 
-{-# LANGUAGE CPP                 #-}
-{-# LANGUAGE DeriveDataTypeable  #-}
-{-# LANGUAGE DeriveGeneric       #-}
-{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE CPP                  #-}
+{-# LANGUAGE DeriveDataTypeable   #-}
+{-# LANGUAGE DeriveGeneric        #-}
+{-# LANGUAGE FlexibleContexts     #-}
+{-# LANGUAGE ScopedTypeVariables  #-}
+{-# LANGUAGE StandaloneDeriving   #-}
+{-# LANGUAGE UndecidableInstances #-}
 
 module Text.Megaparsec.Error.Builder
   ( -- * Top-level helpers
     err
   , errFancy
-    -- * Error position
-  , posI
-  , posN
     -- * Error components
   , utok
   , utoks
@@ -47,14 +46,10 @@ import Data.Set (Set)
 import Data.Typeable (Typeable)
 import GHC.Generics
 import Text.Megaparsec.Error
-import Text.Megaparsec.Pos
 import Text.Megaparsec.Stream
 import qualified Data.List.NonEmpty as NE
 import qualified Data.Set           as E
 
-#if !MIN_VERSION_base(4,8,0)
-import Control.Applicative
-#endif
 #if !MIN_VERSION_base(4,11,0)
 import Data.Semigroup
 #endif
@@ -64,10 +59,19 @@ import Data.Semigroup
 
 -- | Auxiliary type for construction of trivial parse errors.
 
-data ET t = ET (Maybe (ErrorItem t)) (Set (ErrorItem t))
-  deriving (Eq, Ord, Data, Typeable, Generic)
+data ET s = ET (Maybe (ErrorItem (Token s))) (Set (ErrorItem (Token s)))
+  deriving (Typeable, Generic)
 
-instance Ord t => Semigroup (ET t) where
+deriving instance Eq (Token s) => Eq (ET s)
+
+deriving instance Ord (Token s) => Ord (ET s)
+
+deriving instance ( Data s
+                  , Data (Token s)
+                  , Ord (Token s)
+                  ) => Data (ET s)
+
+instance Stream s => Semigroup (ET s) where
   ET us0 ps0 <> ET us1 ps1 = ET (n us0 us1) (E.union ps0 ps1)
     where
       n Nothing  Nothing = Nothing
@@ -75,13 +79,13 @@ instance Ord t => Semigroup (ET t) where
       n Nothing (Just y) = Just y
       n (Just x) (Just y) = Just (max x y)
 
-instance Ord t => Monoid (ET t) where
+instance Stream s => Monoid (ET s) where
   mempty  = ET Nothing E.empty
   mappend = (<>)
 
 -- | Auxiliary type for construction of fancy parse errors.
 
-data EF e = EF (Set (ErrorFancy e))
+newtype EF e = EF (Set (ErrorFancy e))
   deriving (Eq, Ord, Data, Typeable, Generic)
 
 instance Ord e => Semigroup (EF e) where
@@ -94,91 +98,73 @@ instance Ord e => Monoid (EF e) where
 ----------------------------------------------------------------------------
 -- Top-level helpers
 
--- | Assemble a 'ParseError' from source position and @'ET' t@ value. To
--- create source position, two helpers are available: 'posI' and 'posN'.
--- @'ET' t@ is a monoid and can be assembled by combining primitives
--- provided by this module, see below.
+-- | Assemble a 'ParseError' from offset and @'ET' t@ value. @'ET' t@ is a
+-- monoid and can be assembled by combining primitives provided by this
+-- module, see below.
 
 err
-  :: NonEmpty SourcePos -- ^ 'ParseError' position
-  -> ET t              -- ^ Error components
-  -> ParseError t e    -- ^ Resulting 'ParseError'
-err pos (ET us ps) = TrivialError pos us ps
+  :: Int               -- ^ 'ParseError' offset
+  -> ET s              -- ^ Error components
+  -> ParseError s e    -- ^ Resulting 'ParseError'
+err p (ET us ps) = TrivialError p us ps
 
 -- | Like 'err', but constructs a “fancy” 'ParseError'.
 
 errFancy
-  :: NonEmpty SourcePos -- ^ 'ParseError' position
+  :: Int               -- ^ 'ParseError' offset
   -> EF e              -- ^ Error components
-  -> ParseError t e    -- ^ Resulting 'ParseError'
-errFancy pos (EF xs) = FancyError pos xs
-
-----------------------------------------------------------------------------
--- Error position
-
--- | Initial source position with empty file name.
-
-posI :: NonEmpty SourcePos
-posI = initialPos "" :| []
-
--- | @'posN' n s@ returns source position achieved by applying 'advanceN'
--- method corresponding to the type of stream @s@.
-
-posN :: forall s. Stream s
-  => Int
-  -> s
-  -> NonEmpty SourcePos
-posN n s =
-  case takeN_ n s of
-    Nothing -> posI
-    Just (ts, _) ->
-      advanceN (Proxy :: Proxy s) defaultTabWidth (initialPos "") ts :| []
+  -> ParseError s e    -- ^ Resulting 'ParseError'
+errFancy p (EF xs) = FancyError p xs
 
 ----------------------------------------------------------------------------
 -- Error components
 
 -- | Construct an “unexpected token” error component.
 
-utok :: Ord t => t -> ET t
+utok :: Stream s => Token s -> ET s
 utok = unexp . Tokens . nes
 
--- | Construct an “unexpected tokens” error component. Empty string produces
+-- | Construct an “unexpected tokens” error component. Empty chunk produces
 -- 'EndOfInput'.
 
-utoks :: Ord t => [t] -> ET t
-utoks = unexp . canonicalizeTokens
+utoks :: forall s. Stream s => Tokens s -> ET s
+utoks = unexp . canonicalizeTokens (Proxy :: Proxy s)
 
 -- | Construct an “unexpected label” error component. Do not use with empty
 -- strings (for empty strings it's bottom).
 
-ulabel :: Ord t => String -> ET t
-ulabel = unexp . Label . NE.fromList
+ulabel :: Stream s => String -> ET s
+ulabel label
+  | label == "" = error "Text.Megaparsec.Error.Builder.ulabel: empty label"
+  | otherwise = unexp . Label . NE.fromList $ label
 
 -- | Construct an “unexpected end of input” error component.
 
-ueof :: Ord t => ET t
+ueof :: Stream s => ET s
 ueof = unexp EndOfInput
 
 -- | Construct an “expected token” error component.
 
-etok :: Ord t => t -> ET t
+etok :: Stream s => Token s -> ET s
 etok = expe . Tokens . nes
 
--- | Construct an “expected tokens” error component. Empty string produces
+-- | Construct an “expected tokens” error component. Empty chunk produces
 -- 'EndOfInput'.
 
-etoks :: Ord t => [t] -> ET t
-etoks = expe . canonicalizeTokens
+etoks :: forall s. Stream s => Tokens s -> ET s
+etoks = expe . canonicalizeTokens (Proxy :: Proxy s)
 
 -- | Construct an “expected label” error component. Do not use with empty
 -- strings.
 
-elabel :: Ord t => String -> ET t
-elabel = expe . Label . NE.fromList
+elabel :: Stream s => String -> ET s
+elabel label
+  | label == "" = error "Text.Megaparsec.Error.Builder.elabel: empty label"
+  | otherwise = expe . Label . NE.fromList $ label
 
 -- | Construct an “expected end of input” error component.
 
-eeof :: Ord t => ET t
+eeof :: Stream s => ET s
 eeof = expe EndOfInput
 
 -- | Construct a custom error component.
@@ -192,20 +178,24 @@ fancy = EF . E.singleton
 -- | Construct appropriate 'ErrorItem' representation for given token
 -- stream. Empty string produces 'EndOfInput'.
 
-canonicalizeTokens :: [t] -> ErrorItem t
-canonicalizeTokens ts =
-  case NE.nonEmpty ts of
+canonicalizeTokens
+  :: Stream s
+  => Proxy s
+  -> Tokens s
+  -> ErrorItem (Token s)
+canonicalizeTokens pxy ts =
+  case NE.nonEmpty (chunkToTokens pxy ts) of
     Nothing -> EndOfInput
     Just xs -> Tokens xs
 
 -- | Lift an unexpected item into 'ET'.
 
-unexp :: ErrorItem t -> ET t
+unexp :: Stream s => ErrorItem (Token s) -> ET s
 unexp u = ET (pure u) E.empty
 
 -- | Lift an expected item into 'ET'.
 
-expe :: ErrorItem t -> ET t
+expe :: Stream s => ErrorItem (Token s) -> ET s
 expe p = ET Nothing (E.singleton p)
 
 -- | Make a singleton non-empty list from a value.
